@@ -50,7 +50,7 @@ dri2CreateDrawable(VADriverContextP ctx, XID x_drawable)
     dri2_drawable->base.x_drawable = x_drawable;
     dri2_drawable->base.x = 0;
     dri2_drawable->base.y = 0;
-    VA_DRI2CreateDrawable((Display *)ctx->native_dpy, x_drawable);
+    VA_DRI2CreateDrawable(ctx->native_dpy, x_drawable);
 
     return &dri2_drawable->base;
 }
@@ -58,7 +58,7 @@ dri2CreateDrawable(VADriverContextP ctx, XID x_drawable)
 static void 
 dri2DestroyDrawable(VADriverContextP ctx, struct dri_drawable *dri_drawable)
 {
-    VA_DRI2DestroyDrawable((Display *)ctx->native_dpy, dri_drawable->x_drawable);
+    VA_DRI2DestroyDrawable(ctx->native_dpy, dri_drawable->x_drawable);
     free(dri_drawable);
 }
 
@@ -138,14 +138,15 @@ dri2GetRenderingBuffer(VADriverContextP ctx, struct dri_drawable *dri_drawable)
     return &dri2_drawable->buffers[dri2_drawable->front_index];
 }
 
-static void
+void
 dri2Close(VADriverContextP ctx)
 {
     struct dri_state *dri_state = (struct dri_state *)ctx->dri_state;
 
     free_drawable_hashtable(ctx);
-    assert(dri_state->fd >= 0);
-    close(dri_state->fd);
+
+    if (dri_state->fd >= 0);
+	close(dri_state->fd);
 }
 
 Bool 
@@ -160,14 +161,14 @@ isDRI2Connected(VADriverContextP ctx, char **driver_name)
     *driver_name = NULL;
     dri_state->fd = -1;
     dri_state->driConnectedFlag = VA_NONE;
-    if (!VA_DRI2QueryExtension((Display *)ctx->native_dpy, &event_base, &error_base))
+    if (!VA_DRI2QueryExtension(ctx->native_dpy, &event_base, &error_base))
         goto err_out;
 
-    if (!VA_DRI2QueryVersion((Display *)ctx->native_dpy, &major, &minor))
+    if (!VA_DRI2QueryVersion(ctx->native_dpy, &major, &minor))
         goto err_out;
 
 
-    if (!VA_DRI2Connect((Display *)ctx->native_dpy, RootWindow((Display *)ctx->native_dpy, ctx->x11_screen),
+    if (!VA_DRI2Connect(ctx->native_dpy, RootWindow(ctx->native_dpy, ctx->x11_screen),
                      driver_name, &device_name))
         goto err_out;
 
@@ -180,7 +181,7 @@ isDRI2Connected(VADriverContextP ctx, char **driver_name)
     if (drmGetMagic(dri_state->fd, &magic))
         goto err_out;
 
-    if (!VA_DRI2Authenticate((Display *)ctx->native_dpy, RootWindow((Display *)ctx->native_dpy, ctx->x11_screen),
+    if (!VA_DRI2Authenticate(ctx->native_dpy, RootWindow(ctx->native_dpy, ctx->x11_screen),
                           magic))
         goto err_out;
 
@@ -192,6 +193,9 @@ isDRI2Connected(VADriverContextP ctx, char **driver_name)
     dri_state->close = dri2Close;
     gsDRI2SwapAvailable = (minor >= 2);
 
+    if (device_name)
+        Xfree(device_name);
+
     return True;
 
 err_out:
